/**
 * GMyth Library
 * 
 * @file gmyth/gmyth_transcode.c
 * 
 * @brief <p> This file contains the transcode class.
 *
 * Copyright (C) 2007 INdT - Instituto Nokia de Tecnologia.
 * @author Artur Duque de Souza <artur.souza@indt.org.br>
 *
 */
/*
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

 
#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include <stdlib.h>
#include <string.h>
#include <assert.h>

#include "gmyth_transcode.h"
#include "gmyth_util.h"
#include "gmyth_debug.h"

static void gmyth_transcode_class_init          (GMythTranscodeClass *klass);
static void gmyth_transcode_init                (GMythTranscode *object);

static void gmyth_transcode_dispose  (GObject *object);
static void gmyth_transcode_finalize (GObject *object);

G_DEFINE_TYPE(GMythTranscode, gmyth_transcode, G_TYPE_OBJECT)

static void
gmyth_transcode_class_init (GMythTranscodeClass *klass)
{
    GObjectClass *gobject_class = G_OBJECT_CLASS (klass);
    gobject_class->dispose  = gmyth_transcode_dispose;
    gobject_class->finalize = gmyth_transcode_finalize;
}

static void
gmyth_transcode_init (GMythTranscode *transcode)
{
}

static void
gmyth_transcode_dispose  (GObject *object)
{
    GMythTranscode *transcode = GMYTH_TRANSCODE(object);

    if (transcode->output_filename)
        g_free(transcode->output_filename);

    if (transcode->filename)
        g_free(transcode->filename);

    if (transcode->profile)
        g_free(transcode->profile);

    if (transcode->starttime)
        g_free(transcode->starttime);

	G_OBJECT_CLASS (gmyth_transcode_parent_class)->dispose (object);
}

static void
gmyth_transcode_finalize (GObject *object)
{
    g_signal_handlers_destroy (object);
    G_OBJECT_CLASS (gmyth_transcode_parent_class)->finalize (object);
}

/**
 * Creates a new instance of GMythTranscode.
 * 
 * @return a new instance of GMythTranscode.
 **/
GMythTranscode*
gmyth_transcode_new (void)
{
    GMythTranscode *transcode = GMYTH_TRANSCODE\
                                 (g_object_new(GMYTH_TRANSCODE_TYPE, NULL));

    return transcode;
}

/**
 *
 * gmyth_transcode_date_change_format
 * @brief converts a string like YYYY-MM-DDTHH:MM:SS into YYYYMMDDHHMMSS (vice versa)
 * @param date_s gchar*
 * @return gchar* with file or iso format
 *
 **/
gchar* gmyth_transcode_date_change_format (gchar* date_s, int format)
{
    if (date_s != NULL)
    {
        gint length = strlen(date_s);

        //create the right date format
        gchar* src = (gchar*)g_malloc0(sizeof(gchar) * length);
        strncpy(src, date_s, length);

        gchar* dst;

        if (format == DATE_FILE)
        {
            dst = (gchar*)g_malloc0(sizeof(gchar) * 16);
            snprintf(dst, 16, "%.4s%.2s%.2s%.2s%.2s%.2s", src, src+5,\
                    src+7, src+9, src+11, src+13);
            dst[15] = '\0';
        }
        else
        if (format == DATE_ISO)
        {
            dst = (gchar*)g_malloc0(sizeof(gchar) * 20);
            snprintf(dst, 20, "%.4s-%.2s-%.2sT%.2s:%.2s:%.2s", src, src+4,\
                    src+6, src+8, src+10, src+12);
            dst[19] = '\0';
        }

        gchar* ret = g_strdup(dst);

        g_free(src);
        g_free(dst);

        return ret;
    }
    else return NULL;
}

/**
 *
 * gmyth_transcode_set_output
 * @brief set transcode to use output
 * @param value gboolean
 * @param outfile filename of output
 * @return void set's up the var to value
 *
 **/
void gmyth_transcode_set_output (GMythTranscode* transcode,
                                 gboolean value, gchar* outputfile)
{
    transcode->output = value;
    transcode->output_filename = g_strdup(outputfile);
}

/**
 *
 * gmyth_transcode_set_file
 * @brief set the file to transcode
 * @param file filename
 * @return void set's up the var to value
 *
 **/
void gmyth_transcode_set_filename (GMythTranscode* transcode, gchar* file)
{
    if (file != NULL)
    {
        gchar** splited = g_strsplit(file, "_", 2);

        // Get chanid
        sscanf (splited[0],"%d", &(transcode->chanid));

        // Get starttime
        gchar** date = g_strsplit(splited[1], ".", 2);
        transcode->starttime = gmyth_transcode_date_change_format(date[0], DATE_ISO);

        transcode->filename = g_strdup(file);
    }
}


/**
 *
 * gmyth_transcode_set_profile
 * @brief set transcode's profile
 * @param rec GMythTranscode*
 * @param value the value
 * @return gint representing the result
 *
 **/
gint gmyth_transcode_set_profile (GMythTranscode* trans,
                                  gchar* value)
{
    if (value != NULL)
        trans->profile = g_strndup(value, strlen(value));
    else 
        return -1;

    return 0;
}
